@@ -28,45 +28,45 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <glib-object.h>
-#define DBUS_API_SUBJECT_TO_CHANGE
-#include <dbus/dbus-glib.h>
-#include <dbus/dbus-glib-lowlevel.h>
-#include <gconf/gconf-client.h>
+#include <gio/gio.h>
 
 #include "gnome-settings-plugin-info.h"
 #include "gnome-settings-manager.h"
-#include "gnome-settings-manager-glue.h"
 #include "gnome-settings-profile.h"
 
 #define GSD_MANAGER_DBUS_PATH "/org/gnome/SettingsDaemon"
+#define GSD_MANAGER_DBUS_NAME "org.gnome.SettingsDaemon"
 
-#define DEFAULT_SETTINGS_PREFIX "/apps/gnome_settings_daemon/plugins"
+#define DEFAULT_SETTINGS_PREFIX "org.gnome.settings-daemon"
 
 #define PLUGIN_EXT ".gnome-settings-plugin"
 
 #define GNOME_SETTINGS_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GNOME_TYPE_SETTINGS_MANAGER, GnomeSettingsManagerPrivate))
 
+static const gchar introspection_xml[] =
+"<node name='/org/gnome/SettingsDaemon'>"
+"  <interface name='org.gnome.SettingsDaemon'>"
+"    <annotation name='org.freedesktop.DBus.GLib.CSymbol' value='gnome_settings_manager'/>"
+"    <method name='Awake'/>"
+"    <method name='Start'/>"
+"    <signal name='PluginActivated'>"
+"      <arg name='name' type='s'/>"
+"    </signal>"
+"    <signal name='PluginDeactivated'>"
+"      <arg name='name' type='s'/>"
+"    </signal>"
+"  </interface>"
+"</node>";
+
 struct GnomeSettingsManagerPrivate
 {
-        DBusGConnection            *connection;
-        GConfClient                *gconf_client;
-        char                       *settings_prefix;
+        guint                       owner_id;
+        GDBusNodeInfo              *introspection_data;
+        GDBusConnection            *connection;
+        GSettings                  *settings;
         GSList                     *plugins;
 };
 
-enum {
-        PROP_0,
-        PROP_SETTINGS_PREFIX,
-};
-
-enum {
-        PLUGIN_ACTIVATED,
-        PLUGIN_DEACTIVATED,
-        LAST_SIGNAL
-};
-
-static guint signals [LAST_SIGNAL] = { 0, };
-
 static void     gnome_settings_manager_class_init  (GnomeSettingsManagerClass *klass);
 static void     gnome_settings_manager_init        (GnomeSettingsManager      *settings_manager);
 static void     gnome_settings_manager_finalize    (GObject                   *object);
@@ -132,14 +132,40 @@ compare_priority (GnomeSettingsPluginInfo *a,
         return prio_a - prio_b;
 }
 
+static void
+emit_signal (GnomeSettingsManager    *manager,
+             const char              *signal,
+             const char              *name)
+{
+        GError *error = NULL;
+
+        /* FIXME: maybe we should queue those up until the D-Bus
+         * connection is available... */
+        if (manager->priv->connection == NULL)
+                return;
+
+        if (g_dbus_connection_emit_signal (manager->priv->connection,
+                                           NULL,
+                                           GSD_MANAGER_DBUS_PATH,
+                                           GSD_MANAGER_DBUS_NAME,
+                                           "PluginActivated",
+                                           g_variant_new ("(s)", name),
+                                           &error) == FALSE) {
+                g_debug ("Error emitting signal: %s", error->message);
+                g_error_free (error);
+        }
+
+}
+
 static void
 on_plugin_activated (GnomeSettingsPluginInfo *info,
                      GnomeSettingsManager    *manager)
 {
         const char *name;
+
         name = gnome_settings_plugin_info_get_location (info);
         g_debug ("GnomeSettingsManager: emitting plugin-activated %s", name);
-        g_signal_emit (manager, signals [PLUGIN_ACTIVATED], 0, name);
+        emit_signal (manager, "PluginActivated", name);
 }
 
 static void
@@ -147,19 +173,38 @@ on_plugin_deactivated (GnomeSettingsPluginInfo *info,
                        GnomeSettingsManager    *manager)
 {
         const char *name;
+
         name = gnome_settings_plugin_info_get_location (info);
         g_debug ("GnomeSettingsManager: emitting plugin-deactivated %s", name);
-        g_signal_emit (manager, signals [PLUGIN_DEACTIVATED], 0, name);
+        emit_signal (manager, "PluginDeactivated", name);
 }
 
+static gboolean
+contained (const char * const *items,
+           const char         *item)
+{
+        while (*items) {
+                if (g_strcmp0 (*items++, item) == 0) {
+                        return TRUE;
+                }
+        }
+
+        return FALSE;
+}
+
+static gboolean
+is_schema (const char *schema)
+{
+        return contained (g_settings_list_schemas (), schema);
+}
+
+
 static void
 _load_file (GnomeSettingsManager *manager,
             const char           *filename)
 {
         GnomeSettingsPluginInfo *info;
         char                    *key_name;
-        int                      priority;
-        GError                  *error;
         GSList                  *l;
 
         g_debug ("Loading plugin: %s", filename);
@@ -177,32 +222,26 @@ _load_file (GnomeSettingsManager *manager,
                 goto out;
         }
 
-        manager->priv->plugins = g_slist_prepend (manager->priv->plugins,
-                                                  g_object_ref (info));
-
-        g_signal_connect (info, "activated",
-                          G_CALLBACK (on_plugin_activated), manager);
-        g_signal_connect (info, "deactivated",
-                          G_CALLBACK (on_plugin_deactivated), manager);
-
-        key_name = g_strdup_printf ("%s/%s/active",
-                                    manager->priv->settings_prefix,
+        key_name = g_strdup_printf ("%s.plugins.%s",
+                                    DEFAULT_SETTINGS_PREFIX,
                                     gnome_settings_plugin_info_get_location (info));
-        gnome_settings_plugin_info_set_enabled_key_name (info, key_name);
-        g_free (key_name);
 
-        key_name = g_strdup_printf ("%s/%s/priority",
-                                    manager->priv->settings_prefix,
-                                    gnome_settings_plugin_info_get_location (info));
-        error = NULL;
-        priority = gconf_client_get_int (manager->priv->gconf_client, key_name, &error);
-        if (error == NULL) {
-                if (priority > 0) {
-                        gnome_settings_plugin_info_set_priority (info, priority);
-                }
+        /* Ignore unknown schemas or else we'll assert */
+        if (is_schema (key_name)) {
+                manager->priv->plugins = g_slist_prepend (manager->priv->plugins,
+                                                          g_object_ref (info));
+
+                g_signal_connect (info, "activated",
+                                  G_CALLBACK (on_plugin_activated), manager);
+                g_signal_connect (info, "deactivated",
+                                  G_CALLBACK (on_plugin_deactivated), manager);
+
+                gnome_settings_plugin_info_set_settings_prefix (info, key_name);
         } else {
-                g_error_free (error);
+                g_warning ("Ignoring unknown module '%s'", key_name);
         }
+
+        /* Priority is set in the call above */
         g_free (key_name);
 
  out:
@@ -296,23 +335,73 @@ gnome_settings_manager_awake (GnomeSettingsManager *manager,
         return gnome_settings_manager_start (manager, error);
 }
 
-static gboolean
-register_manager (GnomeSettingsManager *manager)
+static void
+handle_method_call (GDBusConnection       *connection,
+                    const gchar           *sender,
+                    const gchar           *object_path,
+                    const gchar           *interface_name,
+                    const gchar           *method_name,
+                    GVariant              *parameters,
+                    GDBusMethodInvocation *invocation,
+                    gpointer               user_data)
 {
+        GnomeSettingsManager *manager = (GnomeSettingsManager *) user_data;
         GError *error = NULL;
 
-        manager->priv->connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
-        if (manager->priv->connection == NULL) {
-                if (error != NULL) {
-                        g_critical ("error getting system bus: %s", error->message);
-                        g_error_free (error);
-                }
-                return FALSE;
+        g_debug ("Calling method '%s' for settings daemon", method_name);
+
+        if (g_strcmp0 (method_name, "Awake") == 0) {
+                if (gnome_settings_manager_awake (manager, &error) == FALSE)
+                        g_dbus_method_invocation_return_gerror (invocation, error);
+                else
+                        g_dbus_method_invocation_return_value (invocation, NULL);
+        } else if (g_strcmp0 (method_name, "Start") == 0) {
+                if (gnome_settings_manager_start (manager, &error) == FALSE)
+                        g_dbus_method_invocation_return_gerror (invocation, error);
+                else
+                        g_dbus_method_invocation_return_value (invocation, NULL);
         }
+}
 
-        dbus_g_connection_register_g_object (manager->priv->connection, GSD_MANAGER_DBUS_PATH, G_OBJECT (manager));
+static const GDBusInterfaceVTable interface_vtable =
+{
+        handle_method_call,
+        NULL, /* Get Property */
+        NULL, /* Set Property */
+};
 
-        return TRUE;
+static void
+on_bus_acquired (GDBusConnection      *connection,
+                 const gchar          *name,
+                 GnomeSettingsManager *manager)
+{
+        guint registration_id;
+
+        registration_id = g_dbus_connection_register_object (connection,
+                                                             GSD_MANAGER_DBUS_PATH,
+                                                             manager->priv->introspection_data->interfaces[0],
+                                                             &interface_vtable,
+                                                             manager,
+                                                             NULL,
+                                                             NULL);
+        if (registration_id > 0)
+                manager->priv->connection = connection;
+}
+
+static void
+register_manager (GnomeSettingsManager *manager)
+{
+        manager->priv->introspection_data = g_dbus_node_info_new_for_xml (introspection_xml, NULL);
+        g_assert (manager->priv->introspection_data != NULL);
+
+        manager->priv->owner_id = g_bus_own_name (G_BUS_TYPE_SESSION,
+                                                  GSD_MANAGER_DBUS_NAME,
+                                                  G_BUS_NAME_OWNER_FLAGS_NONE,
+                                                  (GBusAcquiredCallback) on_bus_acquired,
+                                                  NULL,
+                                                  NULL,
+                                                  manager,
+                                                  NULL);
 }
 
 gboolean
@@ -337,16 +426,11 @@ gnome_settings_manager_start (GnomeSettingsManager *manager,
                 goto out;
         }
 
-        manager->priv->gconf_client = gconf_client_get_default ();
-
-        gnome_settings_profile_start ("preloading gconf keys");
-        gconf_client_add_dir (manager->priv->gconf_client,
-                              manager->priv->settings_prefix,
-                              GCONF_CLIENT_PRELOAD_RECURSIVE,
-                              NULL);
-        gnome_settings_profile_end ("preloading gconf keys");
+        gnome_settings_profile_start ("initializing plugins");
+        manager->priv->settings = g_settings_new (DEFAULT_SETTINGS_PREFIX ".plugins");
 
         _load_all (manager);
+        gnome_settings_profile_end ("initializing plugins");
 
         ret = TRUE;
  out:
@@ -355,85 +439,20 @@ gnome_settings_manager_start (GnomeSettingsManager *manager,
         return ret;
 }
 
-gboolean
-gnome_settings_manager_start_with_settings_prefix (GnomeSettingsManager *manager,
-                                                   const char           *settings_prefix,
-                                                   GError              **error)
-{
-        g_object_set (manager, "settings-prefix", settings_prefix, NULL);
-
-        return gnome_settings_manager_start (manager, error);
-}
-
 void
 gnome_settings_manager_stop (GnomeSettingsManager *manager)
 {
         g_debug ("Stopping settings manager");
 
-#ifdef ENABLE_PYTHON
-        /* Note: that this may cause finalization of objects by
-         * running the garbage collector. Since some of the plugin may
-         * have installed callbacks upon object finalization it must
-         * run before we get rid of the plugins.
-         */
-        gnome_settings_python_shutdown ();
-#endif
-
         _unload_all (manager);
 
-        gconf_client_remove_dir (manager->priv->gconf_client,
-                                 manager->priv->settings_prefix,
-                                 NULL);
-        g_object_unref (manager->priv->gconf_client);
-        manager->priv->gconf_client = NULL;
-}
-
-static void
-_set_settings_prefix (GnomeSettingsManager *self,
-                   const char           *prefix)
-{
-        g_free (self->priv->settings_prefix);
-        self->priv->settings_prefix = g_strdup (prefix);
-}
-
-static void
-gnome_settings_manager_set_property (GObject        *object,
-                                     guint           prop_id,
-                                     const GValue   *value,
-                                     GParamSpec     *pspec)
-{
-        GnomeSettingsManager *self;
-
-        self = GNOME_SETTINGS_MANAGER (object);
-
-        switch (prop_id) {
-        case PROP_SETTINGS_PREFIX:
-                _set_settings_prefix (self, g_value_get_string (value));
-                break;
-        default:
-                G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-                break;
+        if (manager->priv->owner_id > 0) {
+                g_bus_unown_name (manager->priv->owner_id);
+                manager->priv->owner_id = 0;
         }
-}
 
-static void
-gnome_settings_manager_get_property (GObject        *object,
-                                     guint           prop_id,
-                                     GValue         *value,
-                                     GParamSpec     *pspec)
-{
-        GnomeSettingsManager *self;
-
-        self = GNOME_SETTINGS_MANAGER (object);
-
-        switch (prop_id) {
-        case PROP_SETTINGS_PREFIX:
-                g_value_set_string (value, self->priv->settings_prefix);
-                break;
-        default:
-                G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-                break;
-        }
+        g_object_unref (manager->priv->settings);
+        manager->priv->settings = NULL;
 }
 
 static GObject *
@@ -470,44 +489,11 @@ gnome_settings_manager_class_init (GnomeSettingsManagerClass *klass)
 {
         GObjectClass   *object_class = G_OBJECT_CLASS (klass);
 
-        object_class->get_property = gnome_settings_manager_get_property;
-        object_class->set_property = gnome_settings_manager_set_property;
         object_class->constructor = gnome_settings_manager_constructor;
         object_class->dispose = gnome_settings_manager_dispose;
         object_class->finalize = gnome_settings_manager_finalize;
 
-        signals [PLUGIN_ACTIVATED] =
-                g_signal_new ("plugin-activated",
-                              G_TYPE_FROM_CLASS (object_class),
-                              G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (GnomeSettingsManagerClass, plugin_activated),
-                              NULL,
-                              NULL,
-                              g_cclosure_marshal_VOID__STRING,
-                              G_TYPE_NONE,
-                              1, G_TYPE_STRING);
-        signals [PLUGIN_DEACTIVATED] =
-                g_signal_new ("plugin-deactivated",
-                              G_TYPE_FROM_CLASS (object_class),
-                              G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (GnomeSettingsManagerClass, plugin_deactivated),
-                              NULL,
-                              NULL,
-                              g_cclosure_marshal_VOID__STRING,
-                              G_TYPE_NONE,
-                              1, G_TYPE_STRING);
-
-        g_object_class_install_property (object_class,
-                                         PROP_SETTINGS_PREFIX,
-                                         g_param_spec_string ("settings-prefix",
-                                                              "settings-prefix",
-                                                              "settings-prefix",
-                                                              DEFAULT_SETTINGS_PREFIX,
-                                                              G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
-
         g_type_class_add_private (klass, sizeof (GnomeSettingsManagerPrivate));
-
-        dbus_g_object_type_install_info (GNOME_TYPE_SETTINGS_MANAGER, &dbus_glib_gnome_settings_manager_object_info);
 }
 
 static void
@@ -529,8 +515,6 @@ gnome_settings_manager_finalize (GObject *object)
 
         g_return_if_fail (manager->priv != NULL);
 
-        g_free (manager->priv->settings_prefix);
-
         G_OBJECT_CLASS (gnome_settings_manager_parent_class)->finalize (object);
 }
 
@@ -540,17 +524,11 @@ gnome_settings_manager_new (void)
         if (manager_object != NULL) {
                 g_object_ref (manager_object);
         } else {
-                gboolean res;
-
                 manager_object = g_object_new (GNOME_TYPE_SETTINGS_MANAGER,
                                                NULL);
                 g_object_add_weak_pointer (manager_object,
                                            (gpointer *) &manager_object);
-                res = register_manager (manager_object);
-                if (! res) {
-                        g_object_unref (manager_object);
-                        return NULL;
-                }
+                register_manager (manager_object);
         }
 
         return GNOME_SETTINGS_MANAGER (manager_object);
