@@ -31,9 +31,8 @@
 #include <glib/gi18n.h>
 #include <glib/gstdio.h>
 #include <gtk/gtk.h>
-
-#include <dbus/dbus-glib.h>
-#include <dbus/dbus-glib-lowlevel.h>
+#include <gio/gio.h>
+#include <libnotify/notify.h>
 
 #include "gnome-settings-manager.h"
 #include "gnome-settings-profile.h"
@@ -44,165 +43,54 @@
 #define GNOME_SESSION_DBUS_OBJECT    "/org/gnome/SessionManager"
 #define GNOME_SESSION_DBUS_INTERFACE "org.gnome.SessionManager"
 
-static char      *gconf_prefix = NULL;
-static gboolean   no_daemon    = FALSE;
 static gboolean   debug        = FALSE;
-static int        daemon_pipe_fds[2];
+static gboolean   do_timed_exit = FALSE;
 static int        term_signal_pipe_fds[2];
+static guint      name_id      = 0;
+static GnomeSettingsManager *manager = NULL;
 
 static GOptionEntry entries[] = {
         {"debug", 0, 0, G_OPTION_ARG_NONE, &debug, N_("Enable debugging code"), NULL },
-        {"no-daemon", 0, 0, G_OPTION_ARG_NONE, &no_daemon, N_("Don't become a daemon"), NULL },
-        {"gconf-prefix", 0, 0, G_OPTION_ARG_STRING, &gconf_prefix, N_("GConf prefix from which to load plugin settings"), NULL},
+        { "timed-exit", 0, 0, G_OPTION_ARG_NONE, &do_timed_exit, N_("Exit after a time (for debugging)"), NULL },
         {NULL}
 };
 
-static DBusGProxy *
-get_bus_proxy (DBusGConnection *connection)
-{
-        DBusGProxy *bus_proxy;
-
-        bus_proxy = dbus_g_proxy_new_for_name (connection,
-                                               DBUS_SERVICE_DBUS,
-                                               DBUS_PATH_DBUS,
-                                               DBUS_INTERFACE_DBUS);
-        return bus_proxy;
-}
-
 static gboolean
-acquire_name_on_proxy (DBusGProxy *bus_proxy)
+timed_exit_cb (void)
 {
-        GError     *error;
-        guint       result;
-        gboolean    res;
-        gboolean    ret;
-
-        ret = FALSE;
-
-        error = NULL;
-        res = dbus_g_proxy_call (bus_proxy,
-                                 "RequestName",
-                                 &error,
-                                 G_TYPE_STRING, GSD_DBUS_NAME,
-                                 G_TYPE_UINT, 0,
-                                 G_TYPE_INVALID,
-                                 G_TYPE_UINT, &result,
-                                 G_TYPE_INVALID);
-        if (! res) {
-                if (error != NULL) {
-                        g_warning ("Failed to acquire %s: %s", GSD_DBUS_NAME, error->message);
-                        g_error_free (error);
-                } else {
-                        g_warning ("Failed to acquire %s", GSD_DBUS_NAME);
-                }
-                goto out;
-        }
-
-        if (result != DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER) {
-                if (error != NULL) {
-                        g_warning ("Failed to acquire %s: %s", GSD_DBUS_NAME, error->message);
-                        g_error_free (error);
-                } else {
-                        g_warning ("Failed to acquire %s", GSD_DBUS_NAME);
-                }
-                goto out;
-        }
-
-        ret = TRUE;
-
- out:
-        return ret;
-}
-
-static DBusHandlerResult
-bus_message_handler (DBusConnection *connection,
-                     DBusMessage    *message,
-                     void           *user_data)
-{
-        if (dbus_message_is_signal (message,
-                                    DBUS_INTERFACE_LOCAL,
-                                    "Disconnected")) {
-                gtk_main_quit ();
-                return DBUS_HANDLER_RESULT_HANDLED;
-        }
-
-        return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
-}
-
-static DBusGConnection *
-get_session_bus (void)
-{
-        GError          *error;
-        DBusGConnection *bus;
-        DBusConnection  *connection;
-
-        error = NULL;
-        bus = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
-        if (bus == NULL) {
-                g_warning ("Couldn't connect to session bus: %s",
-                           error->message);
-                g_error_free (error);
-                goto out;
-        }
-
-        connection = dbus_g_connection_get_connection (bus);
-        dbus_connection_add_filter (connection,
-                                    (DBusHandleMessageFunction)
-                                    bus_message_handler,
-                                    NULL, NULL);
-
-        dbus_connection_set_exit_on_disconnect (connection, FALSE);
-
- out:
-        return bus;
-}
-
-static gboolean
-bus_register (DBusGConnection *bus)
-{
-        DBusGProxy      *bus_proxy;
-        gboolean         ret;
-
-        gnome_settings_profile_start (NULL);
-
-        ret = FALSE;
-
-        bus_proxy = get_bus_proxy (bus);
-
-        if (bus_proxy == NULL) {
-                g_warning ("Could not construct bus_proxy object");
-                goto out;
-        }
-
-        ret = acquire_name_on_proxy (bus_proxy);
-        g_object_unref (bus_proxy);
-
-        if (!ret) {
-                g_warning ("Could not acquire name");
-                goto out;
-        }
-
-        g_debug ("Successfully connected to D-Bus");
-
- out:
-        gnome_settings_profile_end (NULL);
-
-        return ret;
-}
-
-static void
-on_session_over (DBusGProxy *proxy, GnomeSettingsManager *manager)
-{
-        gnome_settings_manager_stop (manager);
         gtk_main_quit ();
+        return FALSE;
 }
 
 static void
-on_term_signal (int signal)
+on_session_over (GDBusProxy *proxy,
+                 gchar      *sender_name,
+                 gchar      *signal_name,
+                 GVariant   *parameters,
+                 gpointer    user_data)
 {
-        /* Wake up main loop to tell it to shutdown */
-        close (term_signal_pipe_fds[1]);
-        term_signal_pipe_fds[1] = -1;
+        if (g_strcmp0 (signal_name, "SessionOver") == 0) {
+                gnome_settings_manager_stop (manager);
+                gtk_main_quit ();
+        }
+}
+
+static void
+got_session_proxy (GObject *source_object,
+                   GAsyncResult *res,
+                   gpointer user_data)
+{
+        GDBusProxy *proxy;
+        GError *error = NULL;
+
+        proxy = g_dbus_proxy_new_finish (res, &error);
+        if (proxy == NULL) {
+                g_debug ("Could not connect to the Session manager: %s", error->message);
+                g_error_free (error);
+        } else {
+                g_signal_connect (G_OBJECT (proxy), "g-signal",
+                                  G_CALLBACK (on_session_over), NULL);
+        }
 }
 
 static gboolean
@@ -223,6 +111,14 @@ on_term_signal_pipe_closed (GIOChannel *source,
         return FALSE;
 }
 
+static void
+on_term_signal (int signal)
+{
+        /* Wake up main loop to tell it to shutdown */
+        close (term_signal_pipe_fds[1]);
+        term_signal_pipe_fds[1] = -1;
+}
+
 static void
 watch_for_term_signal (GnomeSettingsManager *manager)
 {
@@ -242,41 +138,54 @@ watch_for_term_signal (GnomeSettingsManager *manager)
         g_io_channel_unref (channel);
 
         signal (SIGTERM, on_term_signal);
-
 }
 
 static void
-set_session_over_handler (DBusGConnection *bus, GnomeSettingsManager *manager)
+set_session_over_handler (GDBusConnection *bus)
 {
-        DBusGProxy *session_proxy;
-
         g_assert (bus != NULL);
 
-        gnome_settings_profile_start (NULL);
-
-        session_proxy =
-                 dbus_g_proxy_new_for_name (bus,
-                                            GNOME_SESSION_DBUS_NAME,
-                                            GNOME_SESSION_DBUS_OBJECT,
-                                            GNOME_SESSION_DBUS_INTERFACE);
-
-        dbus_g_object_register_marshaller (
-                g_cclosure_marshal_VOID__VOID,
-                G_TYPE_NONE,
-                G_TYPE_INVALID);
-
-        dbus_g_proxy_add_signal (session_proxy,
-                                 "SessionOver",
-                                 G_TYPE_INVALID);
-
-        dbus_g_proxy_connect_signal (session_proxy,
-                                     "SessionOver",
-                                     G_CALLBACK (on_session_over),
-                                     manager,
-                                     NULL);
+        g_dbus_proxy_new (bus,
+                          G_DBUS_PROXY_FLAGS_NONE,
+                          NULL,
+                          GNOME_SESSION_DBUS_NAME,
+                          GNOME_SESSION_DBUS_OBJECT,
+                          GNOME_SESSION_DBUS_INTERFACE,
+                          NULL,
+                          (GAsyncReadyCallback) got_session_proxy,
+                          NULL);
 
         watch_for_term_signal (manager);
-        gnome_settings_profile_end (NULL);
+}
+
+static void
+name_acquired_handler (GDBusConnection *connection,
+                       const gchar *name,
+                       gpointer user_data)
+{
+        set_session_over_handler (connection);
+}
+
+static void
+name_lost_handler (GDBusConnection *connection,
+                   const gchar *name,
+                   gpointer user_data)
+{
+        /* Name was already taken, or the bus went away */
+        gtk_main_quit ();
+}
+
+static void
+bus_register (void)
+{
+        name_id = g_bus_own_name (G_BUS_TYPE_SESSION,
+                                  GSD_DBUS_NAME,
+                                  G_BUS_NAME_OWNER_FLAGS_NONE,
+                                  NULL,
+                                  (GBusNameAcquiredCallback) name_acquired_handler,
+                                  (GBusNameLostCallback) name_lost_handler,
+                                  NULL,
+                                  NULL);
 }
 
 static void
@@ -297,88 +206,6 @@ gsd_log_default_handler (const gchar   *log_domain,
                                unused_data);
 }
 
-
-/* We want the parent process to quit after initializing all plugins,
- * but we have to do all the work in the child process.  We can't
- * initialize in parent and then fork here: that is not clean with
- * X display and DBUS where we would make the connection from one
- * process and continue using it from the other. So, we just make the
- * parent to fork early and wait. */
-
-static void
-daemon_start (void)
-{
-        int child_pid;
-        char buf[1];
-
-        if (no_daemon)
-                return;
-
-        gnome_settings_profile_msg ("forking daemon");
-
-        signal (SIGPIPE, SIG_IGN);
-        if (-1 == pipe (daemon_pipe_fds)) {
-                g_error ("Could not create pipe: %s", g_strerror (errno));
-                exit (EXIT_FAILURE);
-        }
-
-        child_pid = fork ();
-
-        switch (child_pid) {
-        case -1:
-                g_error ("Could not daemonize: %s", g_strerror (errno));
-                exit (EXIT_FAILURE);
-
-        case 0:
-                /* child */
-
-                close (daemon_pipe_fds[0]);
-
-                return;
-
-         default:
-                /* parent */
-
-                close (daemon_pipe_fds[1]);
-
-                /* Wait for child to signal that we are good to go. */
-                read (daemon_pipe_fds[0], buf, 1);
-
-                exit (EXIT_SUCCESS);
-        }
-}
-
-static void
-daemon_detach (void)
-{
-        if (no_daemon)
-                return;
-
-        gnome_settings_profile_msg ("detaching daemon");
-
-        /* disconnect */
-        setsid ();
-        close (0);
-        close (1);
-        open ("/dev/null", O_RDONLY);
-        open ("/dev/null", O_WRONLY);
-
-        /* get outta the way */
-        chdir ("/");
-}
-
-static void
-daemon_terminate_parent (void)
-{
-        if (no_daemon)
-                return;
-
-        gnome_settings_profile_msg ("terminating parent");
-
-        write (daemon_pipe_fds[1], "1", 1);
-        close (daemon_pipe_fds[1]);
-}
-
 static void
 parse_args (int *argc, char ***argv)
 {
@@ -412,13 +239,14 @@ parse_args (int *argc, char ***argv)
 int
 main (int argc, char *argv[])
 {
-        GnomeSettingsManager *manager;
-        DBusGConnection      *bus;
+
         gboolean              res;
         GError               *error;
 
         manager = NULL;
 
+        gdk_enable_multidevice ();
+
         if (!g_thread_supported ()) {
                 g_thread_init (NULL);
         }
@@ -432,31 +260,20 @@ main (int argc, char *argv[])
 
         parse_args (&argc, &argv);
 
-        daemon_start ();
-
         g_type_init ();
 
         gnome_settings_profile_start ("opening gtk display");
         if (! gtk_init_check (NULL, NULL)) {
                 g_warning ("Unable to initialize GTK+");
-                daemon_terminate_parent ();
                 exit (EXIT_FAILURE);
         }
         gnome_settings_profile_end ("opening gtk display");
 
-        daemon_detach ();
-
         g_log_set_default_handler (gsd_log_default_handler, NULL);
 
-        bus = get_session_bus ();
-        if (bus == NULL) {
-                g_warning ("Could not get a connection to the bus");
-                goto out;
-        }
+        notify_init ("gnome-settings-daemon");
 
-        if (! bus_register (bus)) {
-                goto out;
-        }
+        bus_register ();
 
         gnome_settings_profile_start ("gnome_settings_manager_new");
         manager = gnome_settings_manager_new ();
@@ -466,17 +283,11 @@ main (int argc, char *argv[])
                 goto out;
         }
 
-        set_session_over_handler (bus, manager);
-
         /* If we aren't started by dbus then load the plugins
            automatically.  Otherwise, wait for an Awake etc. */
         if (g_getenv ("DBUS_STARTER_BUS_TYPE") == NULL) {
                 error = NULL;
-                if (gconf_prefix != NULL) {
-                        res = gnome_settings_manager_start_with_settings_prefix (manager, gconf_prefix, &error);
-                } else {
-                        res = gnome_settings_manager_start (manager, &error);
-                }
+                res = gnome_settings_manager_start (manager, &error);
                 if (! res) {
                         g_warning ("Unable to start: %s", error->message);
                         g_error_free (error);
@@ -484,15 +295,16 @@ main (int argc, char *argv[])
                 }
         }
 
-        daemon_terminate_parent ();
+        if (do_timed_exit) {
+                g_timeout_add_seconds (30, (GSourceFunc) timed_exit_cb, NULL);
+        }
 
         gtk_main ();
 
- out:
-        g_free (gconf_prefix);
-
-        if (bus != NULL) {
-                dbus_g_connection_unref (bus);
+out:
+        if (name_id > 0) {
+                g_bus_unown_name (name_id);
+                name_id = 0;
         }
 
         if (manager != NULL) {
