@@ -497,7 +497,7 @@ do_media_action (GsdMediaKeysManager *manager)
 static void
 do_logout_action (GsdMediaKeysManager *manager)
 {
-        execute (manager, "gnome-session-save --logout", FALSE, FALSE);
+        execute (manager, "gnome-session-save --logout-dialog", FALSE, FALSE);
 }
 
 static void
@@ -654,7 +654,7 @@ do_sound_action (GsdMediaKeysManager *manager,
                                 gvc_mixer_stream_push_volume (manager->priv->stream);
                                 sound_changed = TRUE;
                         }
-                } else if (!muted) {
+                } else {
                         vol = vol - norm_vol_step;
                         if (gvc_mixer_stream_set_volume (manager->priv->stream, vol) != FALSE) {
                                 gvc_mixer_stream_push_volume (manager->priv->stream);
@@ -884,6 +884,8 @@ gsd_media_player_key_pressed (GsdMediaKeysManager *manager,
         gboolean    have_listeners;
         GError     *error = NULL;
 
+        g_return_val_if_fail (key != NULL, FALSE);
+
         g_debug ("Media key '%s' pressed", key);
 
         have_listeners = (manager->priv->media_players != NULL);
@@ -897,7 +899,7 @@ gsd_media_player_key_pressed (GsdMediaKeysManager *manager,
                                            GSD_MEDIA_KEYS_DBUS_PATH,
                                            GSD_MEDIA_KEYS_DBUS_NAME,
                                            "MediaPlayerKeyPressed",
-                                           g_variant_new ("(ss)", application, key),
+                                           g_variant_new ("(ss)", application ? application : "", key),
                                            &error) == FALSE) {
                 g_debug ("Error emitting signal: %s", error->message);
                 g_error_free (error);
@@ -952,27 +954,27 @@ do_multimedia_player_action (GsdMediaKeysManager *manager,
 
 static void
 on_xrandr_action_call_finished (GObject             *source_object,
-				GAsyncResult        *res,
-				GsdMediaKeysManager *manager)
+                                GAsyncResult        *res,
+                                GsdMediaKeysManager *manager)
 {
         GError *error = NULL;
         GVariant *variant;
         char *action;
 
         action = g_object_get_data (G_OBJECT (source_object),
-				    "gsd-media-keys-manager-xrandr-action");
+                                    "gsd-media-keys-manager-xrandr-action");
 
-	variant = g_dbus_proxy_call_finish (G_DBUS_PROXY (source_object), res, &error);
+        variant = g_dbus_proxy_call_finish (G_DBUS_PROXY (source_object), res, &error);
 
-	g_object_unref (manager->priv->cancellable);
-	manager->priv->cancellable = NULL;
+        g_object_unref (manager->priv->cancellable);
+        manager->priv->cancellable = NULL;
 
-        if (!res) {
+        if (error != NULL) {
                 g_warning ("Unable to call '%s': %s", action, error->message);
                 g_error_free (error);
         } else {
-		g_variant_unref (variant);
-	}
+                g_variant_unref (variant);
+        }
 
         g_free (action);
 }
@@ -1026,6 +1028,36 @@ do_video_rotate_action (GsdMediaKeysManager *manager,
         return FALSE;
 }
 
+static void
+do_toggle_accessibility_key (const char *key)
+{
+	GSettings *settings;
+	gboolean state;
+
+	settings = g_settings_new ("org.gnome.desktop.a11y.applications");
+	state = g_settings_get_boolean (settings, key);
+	g_settings_set_boolean (settings, key, !state);
+	g_object_unref (settings);
+}
+
+static void
+do_magnifier_action (GsdMediaKeysManager *manager)
+{
+	do_toggle_accessibility_key ("screen-magnifier-enabled");
+}
+
+static void
+do_screenreader_action (GsdMediaKeysManager *manager)
+{
+	do_toggle_accessibility_key ("screen-reader-enabled");
+}
+
+static void
+do_on_screen_keyboard_action (GsdMediaKeysManager *manager)
+{
+	do_toggle_accessibility_key ("screen-keyboard-enabled");
+}
+
 static gboolean
 do_action (GsdMediaKeysManager *manager,
            int                  type,
@@ -1118,6 +1150,15 @@ do_action (GsdMediaKeysManager *manager,
 	case ROTATE_VIDEO_KEY:
 		do_video_rotate_action (manager, timestamp);
 		break;
+	case MAGNIFIER_KEY:
+		do_magnifier_action (manager);
+		break;
+	case SCREENREADER_KEY:
+		do_screenreader_action (manager);
+		break;
+	case ON_SCREEN_KEYBOARD_KEY:
+		do_on_screen_keyboard_action (manager);
+		break;
         case HANDLED_KEYS:
                 g_assert_not_reached ();
         /* Note, no default so compiler catches missing keys */
