@@ -37,30 +37,29 @@
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
-#include <gconf/gconf.h>
-#include <gconf/gconf-client.h>
 
 #include "gnome-settings-profile.h"
+#include "gsd-enums.h"
 #include "gsd-xsettings-manager.h"
+#include "gsd-xsettings-gtk.h"
 #include "xsettings-manager.h"
-#ifdef HAVE_FONTCONFIG
 #include "fontconfig-monitor.h"
-#endif /* HAVE_FONTCONFIG */
 
 #define GNOME_XSETTINGS_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GNOME_TYPE_XSETTINGS_MANAGER, GnomeXSettingsManagerPrivate))
 
-#define MOUSE_SETTINGS_DIR     "/desktop/gnome/peripherals/mouse"
-#define GTK_SETTINGS_DIR       "/desktop/gtk"
-#define INTERFACE_SETTINGS_DIR "/desktop/gnome/interface"
-#define SOUND_SETTINGS_DIR     "/desktop/gnome/sound"
-#define GTK_MODULES_DIR        "/apps/gnome_settings_daemon/gtk-modules"
+#define MOUSE_SETTINGS_SCHEMA     "org.gnome.settings-daemon.peripherals.mouse"
+#define INTERFACE_SETTINGS_SCHEMA "org.gnome.desktop.interface"
+#define SOUND_SETTINGS_SCHEMA     "org.gnome.desktop.sound"
 
-#ifdef HAVE_FONTCONFIG
-#define FONT_RENDER_DIR "/desktop/gnome/font_rendering"
-#define FONT_ANTIALIASING_KEY FONT_RENDER_DIR "/antialiasing"
-#define FONT_HINTING_KEY      FONT_RENDER_DIR "/hinting"
-#define FONT_RGBA_ORDER_KEY   FONT_RENDER_DIR "/rgba_order"
-#define FONT_DPI_KEY          FONT_RENDER_DIR "/dpi"
+#define XSETTINGS_PLUGIN_SCHEMA "org.gnome.settings-daemon.plugins.xsettings"
+
+#define GTK_MODULES_DISABLED_KEY "disabled-gtk-modules"
+#define GTK_MODULES_ENABLED_KEY  "enabled-gtk-modules"
+
+#define FONT_ANTIALIASING_KEY "antialiasing"
+#define FONT_HINTING_KEY      "hinting"
+#define FONT_RGBA_ORDER_KEY   "rgba-order"
+#define FONT_DPI_KEY          "dpi"
 
 /* X servers sometimes lie about the screen's physical dimensions, so we cannot
  * compute an accurate DPI value.  When this happens, the user gets fonts that
@@ -69,35 +68,35 @@
  * DPI_HIGH_REASONABLE_VALUE], then we assume that it is lying and we use
  * DPI_FALLBACK instead.
  *
- * See get_dpi_from_gconf_or_server() below, and also
+ * See get_dpi_from_gsettings_or_x_server() below, and also
  * https://bugzilla.novell.com/show_bug.cgi?id=217790
  */
 #define DPI_FALLBACK 96
 #define DPI_LOW_REASONABLE_VALUE 50
 #define DPI_HIGH_REASONABLE_VALUE 500
 
-#endif /* HAVE_FONTCONFIG */
-
 typedef struct _TranslationEntry TranslationEntry;
 typedef void (* TranslationFunc) (GnomeXSettingsManager *manager,
                                   TranslationEntry      *trans,
-                                  GConfValue            *value);
+                                  GVariant              *value);
 
 struct _TranslationEntry {
-        const char     *gconf_key;
+        const char     *gsettings_schema;
+        const char     *gsettings_key;
         const char     *xsetting_name;
 
-        GConfValueType  gconf_type;
         TranslationFunc translate;
 };
 
 struct GnomeXSettingsManagerPrivate
 {
         XSettingsManager **managers;
-        guint              notify[6];
-#ifdef HAVE_FONTCONFIG
+        GHashTable        *settings;
+
+        GSettings         *plugin_settings;
         fontconfig_monitor_handle_t *fontconfig_handle;
-#endif /* HAVE_FONTCONFIG */
+
+        GsdXSettingsGtk   *gtk;
 };
 
 #define GSD_XSETTINGS_ERROR gsd_xsettings_error_quark ()
@@ -123,63 +122,55 @@ gsd_xsettings_error_quark (void)
 static void
 translate_bool_int (GnomeXSettingsManager *manager,
                     TranslationEntry      *trans,
-                    GConfValue            *value)
+                    GVariant              *value)
 {
         int i;
 
-        g_assert (value->type == trans->gconf_type);
-
         for (i = 0; manager->priv->managers [i]; i++) {
                 xsettings_manager_set_int (manager->priv->managers [i], trans->xsetting_name,
-                                           gconf_value_get_bool (value));
+                                           g_variant_get_boolean (value));
         }
 }
 
 static void
 translate_int_int (GnomeXSettingsManager *manager,
                    TranslationEntry      *trans,
-                   GConfValue            *value)
+                   GVariant              *value)
 {
         int i;
 
-        g_assert (value->type == trans->gconf_type);
-
         for (i = 0; manager->priv->managers [i]; i++) {
                 xsettings_manager_set_int (manager->priv->managers [i], trans->xsetting_name,
-                                           gconf_value_get_int (value));
+                                           g_variant_get_int32 (value));
         }
 }
 
 static void
 translate_string_string (GnomeXSettingsManager *manager,
                          TranslationEntry      *trans,
-                         GConfValue            *value)
+                         GVariant              *value)
 {
         int i;
 
-        g_assert (value->type == trans->gconf_type);
-
         for (i = 0; manager->priv->managers [i]; i++) {
                 xsettings_manager_set_string (manager->priv->managers [i],
                                               trans->xsetting_name,
-                                              gconf_value_get_string (value));
+                                              g_variant_get_string (value, NULL));
         }
 }
 
 static void
 translate_string_string_toolbar (GnomeXSettingsManager *manager,
                                  TranslationEntry      *trans,
-                                 GConfValue            *value)
+                                 GVariant              *value)
 {
         int         i;
         const char *tmp;
 
-        g_assert (value->type == trans->gconf_type);
-
         /* This is kind of a workaround since GNOME expects the key value to be
          * "both_horiz" and gtk+ wants the XSetting to be "both-horiz".
          */
-        tmp = gconf_value_get_string (value);
+        tmp = g_variant_get_string (value, NULL);
         if (tmp && strcmp (tmp, "both_horiz") == 0) {
                 tmp = "both-horiz";
         }
@@ -192,36 +183,40 @@ translate_string_string_toolbar (GnomeXSettingsManager *manager,
 }
 
 static TranslationEntry translations [] = {
-        { "/desktop/gnome/peripherals/mouse/double_click",   "Net/DoubleClickTime",     GCONF_VALUE_INT,      translate_int_int },
-        { "/desktop/gnome/peripherals/mouse/drag_threshold", "Net/DndDragThreshold",    GCONF_VALUE_INT,      translate_int_int },
-        { "/desktop/gnome/gtk-color-palette",                "Gtk/ColorPalette",        GCONF_VALUE_STRING,   translate_string_string },
-        { "/desktop/gnome/interface/font_name",              "Gtk/FontName",            GCONF_VALUE_STRING,   translate_string_string },
-        { "/desktop/gnome/interface/gtk_key_theme",          "Gtk/KeyThemeName",        GCONF_VALUE_STRING,   translate_string_string },
-        { "/desktop/gnome/interface/toolbar_style",          "Gtk/ToolbarStyle",        GCONF_VALUE_STRING,   translate_string_string_toolbar },
-        { "/desktop/gnome/interface/toolbar_icons_size",     "Gtk/ToolbarIconSize",     GCONF_VALUE_STRING,   translate_string_string },
-        { "/desktop/gnome/interface/can_change_accels",      "Gtk/CanChangeAccels",     GCONF_VALUE_BOOL,     translate_bool_int },
-        { "/desktop/gnome/interface/cursor_blink",           "Net/CursorBlink",         GCONF_VALUE_BOOL,     translate_bool_int },
-        { "/desktop/gnome/interface/cursor_blink_time",      "Net/CursorBlinkTime",     GCONF_VALUE_INT,      translate_int_int },
-        { "/desktop/gnome/interface/gtk_theme",              "Net/ThemeName",           GCONF_VALUE_STRING,   translate_string_string },
-        { "/desktop/gnome/interface/gtk_color_scheme",       "Gtk/ColorScheme",         GCONF_VALUE_STRING,   translate_string_string },
-        { "/desktop/gnome/interface/gtk-im-preedit-style",   "Gtk/IMPreeditStyle",      GCONF_VALUE_STRING,   translate_string_string },
-        { "/desktop/gnome/interface/gtk-im-status-style",    "Gtk/IMStatusStyle",       GCONF_VALUE_STRING,   translate_string_string },
-        { "/desktop/gnome/interface/gtk-im-module",          "Gtk/IMModule",            GCONF_VALUE_STRING,   translate_string_string },
-        { "/desktop/gnome/interface/icon_theme",             "Net/IconThemeName",       GCONF_VALUE_STRING,   translate_string_string },
-        { "/desktop/gnome/interface/file_chooser_backend",   "Gtk/FileChooserBackend",  GCONF_VALUE_STRING,   translate_string_string },
-        { "/desktop/gnome/interface/menus_have_icons",       "Gtk/MenuImages",          GCONF_VALUE_BOOL,     translate_bool_int },
-        { "/desktop/gnome/interface/buttons_have_icons",     "Gtk/ButtonImages",          GCONF_VALUE_BOOL,     translate_bool_int },
-        { "/desktop/gnome/interface/menubar_accel",          "Gtk/MenuBarAccel",        GCONF_VALUE_STRING,   translate_string_string },
-        { "/desktop/gnome/peripherals/mouse/cursor_theme",   "Gtk/CursorThemeName",     GCONF_VALUE_STRING,   translate_string_string },
-        { "/desktop/gnome/peripherals/mouse/cursor_size",    "Gtk/CursorThemeSize",     GCONF_VALUE_INT,      translate_int_int },
-        { "/desktop/gnome/interface/show_input_method_menu", "Gtk/ShowInputMethodMenu", GCONF_VALUE_BOOL,     translate_bool_int },
-        { "/desktop/gnome/interface/show_unicode_menu",      "Gtk/ShowUnicodeMenu",     GCONF_VALUE_BOOL,     translate_bool_int },
-        { "/desktop/gnome/sound/theme_name",                 "Net/SoundThemeName",      GCONF_VALUE_STRING,   translate_string_string },
-        { "/desktop/gnome/sound/event_sounds",               "Net/EnableEventSounds" ,  GCONF_VALUE_BOOL,     translate_bool_int },
-        { "/desktop/gnome/sound/input_feedback_sounds",      "Net/EnableInputFeedbackSounds", GCONF_VALUE_BOOL, translate_bool_int }
+        { "org.gnome.settings-daemon.peripherals.mouse", "double-click",   "Net/DoubleClickTime",  translate_int_int },
+        { "org.gnome.settings-daemon.peripherals.mouse", "drag-threshold", "Net/DndDragThreshold", translate_int_int },
+
+        { "org.gnome.desktop.interface", "gtk-color-palette",      "Gtk/ColorPalette",        translate_string_string },
+        { "org.gnome.desktop.interface", "font-name",              "Gtk/FontName",            translate_string_string },
+        { "org.gnome.desktop.interface", "gtk-key-theme",          "Gtk/KeyThemeName",        translate_string_string },
+        { "org.gnome.desktop.interface", "toolbar-style",          "Gtk/ToolbarStyle",        translate_string_string_toolbar },
+        { "org.gnome.desktop.interface", "toolbar-icons-size",     "Gtk/ToolbarIconSize",     translate_string_string },
+        { "org.gnome.desktop.interface", "can-change-accels",      "Gtk/CanChangeAccels",     translate_bool_int },
+        { "org.gnome.desktop.interface", "cursor-blink",           "Net/CursorBlink",         translate_bool_int },
+        { "org.gnome.desktop.interface", "cursor-blink-time",      "Net/CursorBlinkTime",     translate_int_int },
+        { "org.gnome.desktop.interface", "cursor-blink-timeout",   "Gtk/CursorBlinkTimeout",  translate_int_int },
+        { "org.gnome.desktop.interface", "gtk-theme",              "Net/ThemeName",           translate_string_string },
+        { "org.gnome.desktop.interface", "gtk-timeout-initial",    "Gtk/TimeoutInitial",      translate_int_int },
+        { "org.gnome.desktop.interface", "gtk-timeout-repeat",     "Gtk/TimeoutRepeat",       translate_int_int },
+        { "org.gnome.desktop.interface", "gtk-color-scheme",       "Gtk/ColorScheme",         translate_string_string },
+        { "org.gnome.desktop.interface", "gtk-im-preedit-style",   "Gtk/IMPreeditStyle",      translate_string_string },
+        { "org.gnome.desktop.interface", "gtk-im-status-style",    "Gtk/IMStatusStyle",       translate_string_string },
+        { "org.gnome.desktop.interface", "gtk-im-module",          "Gtk/IMModule",            translate_string_string },
+        { "org.gnome.desktop.interface", "icon-theme",             "Net/IconThemeName",       translate_string_string },
+        { "org.gnome.desktop.interface", "menus-have-icons",       "Gtk/MenuImages",          translate_bool_int },
+        { "org.gnome.desktop.interface", "buttons-have-icons",     "Gtk/ButtonImages",        translate_bool_int },
+        { "org.gnome.desktop.interface", "menubar-accel",          "Gtk/MenuBarAccel",        translate_string_string },
+        { "org.gnome.desktop.interface", "enable-animations",      "Gtk/EnableAnimations",    translate_bool_int },
+        { "org.gnome.desktop.interface", "cursor-theme",           "Gtk/CursorThemeName",     translate_string_string },
+        { "org.gnome.desktop.interface", "cursor-size",            "Gtk/CursorThemeSize",     translate_int_int },
+        { "org.gnome.desktop.interface", "show-input-method-menu", "Gtk/ShowInputMethodMenu", translate_bool_int },
+        { "org.gnome.desktop.interface", "show-unicode-menu",      "Gtk/ShowUnicodeMenu",     translate_bool_int },
+
+        { "org.gnome.desktop.sound", "theme-name",                 "Net/SoundThemeName",            translate_string_string },
+        { "org.gnome.desktop.sound", "event-sounds",               "Net/EnableEventSounds" ,        translate_bool_int },
+        { "org.gnome.desktop.sound", "input-feedback-sounds",      "Net/EnableInputFeedbackSounds", translate_bool_int }
 };
 
-#ifdef HAVE_FONTCONFIG
 static double
 dpi_from_pixels_and_mm (int pixels,
                         int mm)
@@ -265,31 +260,18 @@ get_dpi_from_x_server (void)
 }
 
 static double
-get_dpi_from_gconf_or_x_server (GConfClient *client)
+get_dpi_from_gsettings_or_x_server (GnomeXSettingsManager *manager)
 {
-        GConfValue *value;
         double      dpi;
 
-        value = gconf_client_get_without_default (client, FONT_DPI_KEY, NULL);
-
-        /* If the user has ever set the DPI preference in GConf, we use that.
-         * Otherwise, we see if the X server reports a reasonable DPI value:  some X
-         * servers report completely bogus values, and the user gets huge or tiny
-         * fonts which are unusable.
-         */
-
-        if (value != NULL) {
-                dpi = gconf_value_get_float (value);
-                gconf_value_free (value);
-        } else {
+        dpi = g_settings_get_double (manager->priv->plugin_settings, FONT_DPI_KEY);
+        if (dpi == 0.0)
                 dpi = get_dpi_from_x_server ();
-        }
 
         return dpi;
 }
 
-typedef struct
-{
+typedef struct {
         gboolean    antialias;
         gboolean    hinting;
         int         dpi;
@@ -297,90 +279,58 @@ typedef struct
         const char *hintstyle;
 } GnomeXftSettings;
 
-static const char *rgba_types[] = { "rgb", "bgr", "vbgr", "vrgb" };
-
-/* Read GConf settings and determine the appropriate Xft settings based on them
- * This probably could be done a bit more cleanly with gconf_string_to_enum
- */
+/* Read GSettings and determine the appropriate Xft settings based on them. */
 static void
-xft_settings_get (GConfClient      *client,
-                  GnomeXftSettings *settings)
+xft_settings_get (GnomeXSettingsManager *manager,
+                  GnomeXftSettings      *settings)
 {
-        char  *antialiasing;
-        char  *hinting;
-        char  *rgba_order;
+        GsdFontAntialiasingMode antialiasing;
+        GsdFontHinting hinting;
+        GsdFontRgbaOrder rgba_order;
         double dpi;
+        gboolean use_rgba = FALSE;
 
-        antialiasing = gconf_client_get_string (client, FONT_ANTIALIASING_KEY, NULL);
-        hinting = gconf_client_get_string (client, FONT_HINTING_KEY, NULL);
-        rgba_order = gconf_client_get_string (client, FONT_RGBA_ORDER_KEY, NULL);
-        dpi = get_dpi_from_gconf_or_x_server (client);
+        antialiasing = g_settings_get_enum (manager->priv->plugin_settings, FONT_ANTIALIASING_KEY);
+        hinting = g_settings_get_enum (manager->priv->plugin_settings, FONT_HINTING_KEY);
+        rgba_order = g_settings_get_enum (manager->priv->plugin_settings, FONT_RGBA_ORDER_KEY);
+        dpi = get_dpi_from_gsettings_or_x_server (manager);
 
-        settings->antialias = TRUE;
-        settings->hinting = TRUE;
-        settings->hintstyle = "hintfull";
+        settings->antialias = (antialiasing != GSD_FONT_ANTIALIASING_MODE_NONE);
+        settings->hinting = (hinting != GSD_FONT_HINTING_NONE);
         settings->dpi = dpi * 1024; /* Xft wants 1/1024ths of an inch */
         settings->rgba = "rgb";
+        settings->hintstyle = "hintfull";
 
-        if (rgba_order) {
-                int i;
-                gboolean found = FALSE;
-
-                for (i = 0; i < G_N_ELEMENTS (rgba_types) && !found; i++) {
-                        if (strcmp (rgba_order, rgba_types[i]) == 0) {
-                                settings->rgba = rgba_types[i];
-                                found = TRUE;
-                        }
-                }
-
-                if (!found) {
-                        g_warning ("Invalid value for " FONT_RGBA_ORDER_KEY ": '%s'",
-                                   rgba_order);
-                }
+        switch (hinting) {
+        case GSD_FONT_HINTING_NONE:
+                settings->hintstyle = "hintnone";
+                break;
+        case GSD_FONT_HINTING_SLIGHT:
+                settings->hintstyle = "hintslight";
+                break;
+        case GSD_FONT_HINTING_MEDIUM:
+                settings->hintstyle = "hintmedium";
+                break;
+        case GSD_FONT_HINTING_FULL:
+                settings->hintstyle = "hintfull";
+                break;
         }
 
-        if (hinting) {
-                if (strcmp (hinting, "none") == 0) {
-                        settings->hinting = 0;
-                        settings->hintstyle = "hintnone";
-                } else if (strcmp (hinting, "slight") == 0) {
-                        settings->hinting = 1;
-                        settings->hintstyle = "hintslight";
-                } else if (strcmp (hinting, "medium") == 0) {
-                        settings->hinting = 1;
-                        settings->hintstyle = "hintmedium";
-                } else if (strcmp (hinting, "full") == 0) {
-                        settings->hinting = 1;
-                        settings->hintstyle = "hintfull";
-                } else {
-                        g_warning ("Invalid value for " FONT_HINTING_KEY ": '%s'",
-                                   hinting);
-                }
+        switch (antialiasing) {
+        case GSD_FONT_ANTIALIASING_MODE_NONE:
+                settings->antialias = 0;
+                break;
+        case GSD_FONT_ANTIALIASING_MODE_GRAYSCALE:
+                settings->antialias = 1;
+                break;
+        case GSD_FONT_ANTIALIASING_MODE_RGBA:
+                settings->antialias = 1;
+                use_rgba = TRUE;
         }
 
-        if (antialiasing) {
-                gboolean use_rgba = FALSE;
-
-                if (strcmp (antialiasing, "none") == 0) {
-                        settings->antialias = 0;
-                } else if (strcmp (antialiasing, "grayscale") == 0) {
-                        settings->antialias = 1;
-                } else if (strcmp (antialiasing, "rgba") == 0) {
-                        settings->antialias = 1;
-                        use_rgba = TRUE;
-                } else {
-                        g_warning ("Invalid value for " FONT_ANTIALIASING_KEY " : '%s'",
-                                   antialiasing);
-                }
-
-                if (!use_rgba) {
-                        settings->rgba = "none";
-                }
+        if (!use_rgba) {
+                settings->rgba = "none";
         }
-
-        g_free (rgba_order);
-        g_free (hinting);
-        g_free (antialiasing);
 }
 
 static void
@@ -397,6 +347,8 @@ xft_settings_set_xsettings (GnomeXSettingsManager *manager,
                 xsettings_manager_set_string (manager->priv->managers [i], "Xft/HintStyle", settings->hintstyle);
                 xsettings_manager_set_int (manager->priv->managers [i], "Xft/DPI", settings->dpi);
                 xsettings_manager_set_string (manager->priv->managers [i], "Xft/RGBA", settings->rgba);
+                xsettings_manager_set_string (manager->priv->managers [i], "Xft/lcdfilter",
+                                              g_str_equal (settings->rgba, "rgb") ? "lcddefault" : "none");
         }
         gnome_settings_profile_end (NULL);
 }
@@ -456,12 +408,14 @@ xft_settings_set_xresources (GnomeXftSettings *settings)
                                 settings->hintstyle);
         update_property (add_string, "Xft.rgba",
                                 settings->rgba);
+        update_property (add_string, "Xft.lcdfilter",
+                         g_str_equal (settings->rgba, "rgb") ? "lcddefault" : "none");
 
         g_debug("xft_settings_set_xresources: new res '%s'", add_string->str);
 
         /* Set the new X property */
         XChangeProperty(dpy, RootWindow (dpy, 0),
-                XA_RESOURCE_MANAGER, XA_STRING, 8, PropModeReplace, add_string->str, add_string->len);
+                        XA_RESOURCE_MANAGER, XA_STRING, 8, PropModeReplace, (const unsigned char *) add_string->str, add_string->len);
         XCloseDisplay (dpy);
 
         g_string_free (add_string, TRUE);
@@ -473,14 +427,13 @@ xft_settings_set_xresources (GnomeXftSettings *settings)
  * X resources
  */
 static void
-update_xft_settings (GnomeXSettingsManager *manager,
-                     GConfClient           *client)
+update_xft_settings (GnomeXSettingsManager *manager)
 {
         GnomeXftSettings settings;
 
         gnome_settings_profile_start (NULL);
 
-        xft_settings_get (client, &settings);
+        xft_settings_get (manager, &settings);
         xft_settings_set_xsettings (manager, &settings);
         xft_settings_set_xresources (&settings);
 
@@ -488,20 +441,58 @@ update_xft_settings (GnomeXSettingsManager *manager,
 }
 
 static void
-xft_callback (GConfClient           *client,
-              guint                  cnxn_id,
-              GConfEntry            *entry,
+xft_callback (GSettings             *settings,
+              const gchar           *key,
               GnomeXSettingsManager *manager)
 {
         int i;
 
-        update_xft_settings (manager, client);
+        update_xft_settings (manager);
 
         for (i = 0; manager->priv->managers [i]; i++) {
                 xsettings_manager_notify (manager->priv->managers [i]);
         }
 }
 
+static void
+plugin_callback (GSettings             *settings,
+                 const char            *key,
+                 GnomeXSettingsManager *manager)
+{
+        if (g_str_equal (key, GTK_MODULES_DISABLED_KEY) ||
+            g_str_equal (key, GTK_MODULES_ENABLED_KEY)) {
+                /* Do nothing, as GsdXsettingsGtk will handle it */
+        } else {
+                xft_callback (settings, key, manager);
+        }
+}
+
+static void
+gtk_modules_callback (GsdXSettingsGtk       *gtk,
+                      GParamSpec            *spec,
+                      GnomeXSettingsManager *manager)
+{
+        const char *modules = gsd_xsettings_gtk_get_modules (manager->priv->gtk);
+        int i;
+
+        if (modules == NULL) {
+                for (i = 0; manager->priv->managers [i]; ++i) {
+                        xsettings_manager_delete_setting (manager->priv->managers [i], "Gtk/Modules");
+                }
+        } else {
+                g_debug ("Setting GTK modules '%s'", modules);
+                for (i = 0; manager->priv->managers [i]; ++i) {
+                        xsettings_manager_set_string (manager->priv->managers [i],
+                                                      "Gtk/Modules",
+                                                      modules);
+                }
+        }
+
+        for (i = 0; manager->priv->managers [i]; ++i) {
+                xsettings_manager_notify (manager->priv->managers [i]);
+        }
+}
+
 static void
 fontconfig_callback (fontconfig_monitor_handle_t *handle,
                      GnomeXSettingsManager       *manager)
@@ -550,86 +541,55 @@ stop_fontconfig_monitor (GnomeXSettingsManager  *manager)
                 manager->priv->fontconfig_handle = NULL;
         }
 }
-#endif /* HAVE_FONTCONFIG */
-
-static const char *
-type_to_string (GConfValueType type)
-{
-        switch (type) {
-        case GCONF_VALUE_INT:
-                return "int";
-        case GCONF_VALUE_STRING:
-                return "string";
-        case GCONF_VALUE_FLOAT:
-                return "float";
-        case GCONF_VALUE_BOOL:
-                return "bool";
-        case GCONF_VALUE_SCHEMA:
-                return "schema";
-        case GCONF_VALUE_LIST:
-                return "list";
-        case GCONF_VALUE_PAIR:
-                return "pair";
-        case GCONF_VALUE_INVALID:
-                return "*invalid*";
-        default:
-                g_assert_not_reached();
-                return NULL; /* for warnings */
-        }
-}
 
 static void
 process_value (GnomeXSettingsManager *manager,
                TranslationEntry      *trans,
-               GConfValue            *val)
+               GVariant              *value)
 {
-        if (val == NULL) {
-                int i;
-
-                for (i = 0; manager->priv->managers [i]; i++) {
-                        xsettings_manager_delete_setting (manager->priv->managers [i], trans->xsetting_name);
-                }
-        } else {
-                if (val->type == trans->gconf_type) {
-                        (* trans->translate) (manager, trans, val);
-                } else {
-                        g_warning (_("GConf key %s set to type %s but its expected type was %s\n"),
-                                   trans->gconf_key,
-                                   type_to_string (val->type),
-                                   type_to_string (trans->gconf_type));
-                }
-        }
+        (* trans->translate) (manager, trans, value);
 }
 
 static TranslationEntry *
-find_translation_entry (const char *gconf_key)
+find_translation_entry (GSettings *settings, const char *key)
 {
-        int i;
+        guint i;
+        char *schema;
 
-        for (i = 0; i < G_N_ELEMENTS (translations); ++i) {
-                if (strcmp (translations[i].gconf_key, gconf_key) == 0) {
+        g_object_get (settings, "schema", &schema, NULL);
+
+        for (i = 0; i < G_N_ELEMENTS (translations); i++) {
+                if (g_str_equal (schema, translations[i].gsettings_schema) &&
+                    g_str_equal (key, translations[i].gsettings_key)) {
+                            g_free (schema);
                         return &translations[i];
                 }
         }
 
+        g_free (schema);
+
         return NULL;
 }
 
 static void
-xsettings_callback (GConfClient           *client,
-                    guint                  cnxn_id,
-                    GConfEntry            *entry,
+xsettings_callback (GSettings             *settings,
+                    const char            *key,
                     GnomeXSettingsManager *manager)
 {
         TranslationEntry *trans;
-        int               i;
+        guint             i;
+        GVariant         *value;
 
-        trans = find_translation_entry (entry->key);
+        trans = find_translation_entry (settings, key);
         if (trans == NULL) {
                 return;
         }
 
-        process_value (manager, trans, entry->value);
+        value = g_settings_get_value (settings, key);
+
+        process_value (manager, trans, value);
+
+        g_object_unref (value);
 
         for (i = 0; manager->priv->managers [i]; i++) {
                 xsettings_manager_set_string (manager->priv->managers [i],
@@ -642,101 +602,6 @@ xsettings_callback (GConfClient           *client,
         }
 }
 
-static gchar *
-get_gtk_modules (GConfClient *client)
-{
-        GSList *entries, *l;
-        GString *mods = g_string_new (NULL);
-
-        entries = gconf_client_all_entries (client, GTK_MODULES_DIR, NULL);
-
-        for (l = entries; l != NULL; l = g_slist_next (l)) {
-                GConfEntry *e = l->data;
-                GConfValue *v = gconf_entry_get_value (e);
-
-                if (v != NULL) {
-                        gboolean enabled = FALSE;
-                        const gchar *key;
-
-                        switch (v->type) {
-                        case GCONF_VALUE_BOOL:
-                                /* simple enabled/disabled */
-                                enabled = gconf_value_get_bool (v);
-                                break;
-
-                        /* due to limitations in GConf (or the client libraries,
-                         * anyway), it is currently impossible to monitor
-                         * arbitrary keys for changes, so these won't update at
-                         * runtime */
-                        case GCONF_VALUE_STRING:
-                                /* linked to another GConf key of type bool */
-                                key = gconf_value_get_string (v);
-                                if (key != NULL && gconf_valid_key (key, NULL)) {
-                                        enabled = gconf_client_get_bool (client, key, NULL);
-                                }
-                                break;
-
-                        default:
-                                g_warning ("GConf entry %s has invalid type %s",
-                                           gconf_entry_get_key (e), type_to_string (v->type));
-                        }
-
-                        if (enabled) {
-                                const gchar *name;
-                                name = strrchr (gconf_entry_get_key (e), '/') + 1;
-
-                                if (mods->len > 0) {
-                                        g_string_append_c (mods, ':');
-                                }
-                                g_string_append (mods, name);
-                        }
-                }
-
-                gconf_entry_free (e);
-        }
-
-        g_slist_free (entries);
-
-        return g_string_free (mods, mods->len == 0);
-}
-
-static void
-gtk_modules_callback (GConfClient           *client,
-                      guint                  cnxn_id,
-                      GConfEntry            *entry,
-                      GnomeXSettingsManager *manager)
-{
-        gchar *modules = get_gtk_modules (client);
-        int i;
-
-        if (modules == NULL) {
-                for (i = 0; manager->priv->managers [i]; ++i) {
-                        xsettings_manager_delete_setting (manager->priv->managers [i], "Gtk/Modules");
-                }
-        } else {
-                g_debug ("Setting GTK modules '%s'", modules);
-                for (i = 0; manager->priv->managers [i]; ++i) {
-                        xsettings_manager_set_string (manager->priv->managers [i],
-                                                      "Gtk/Modules",
-                                                      modules);
-                }
-                g_free (modules);
-        }
-
-        for (i = 0; manager->priv->managers [i]; ++i) {
-                xsettings_manager_notify (manager->priv->managers [i]);
-        }
-}
-
-static guint
-register_config_callback (GnomeXSettingsManager  *manager,
-                          GConfClient            *client,
-                          const char             *path,
-                          GConfClientNotifyFunc   func)
-{
-        return gconf_client_notify_add (client, path, func, manager, NULL, NULL);
-}
-
 static void
 terminate_cb (void *data)
 {
@@ -795,8 +660,8 @@ gboolean
 gnome_xsettings_manager_start (GnomeXSettingsManager *manager,
                                GError               **error)
 {
-        GConfClient *client;
-        int          i;
+        guint        i;
+        GList       *list, *l;
 
         g_debug ("Starting xsettings manager");
         gnome_settings_profile_start (NULL);
@@ -808,71 +673,52 @@ gnome_xsettings_manager_start (GnomeXSettingsManager *manager,
                 return FALSE;
         }
 
-        client = gconf_client_get_default ();
+        manager->priv->settings = g_hash_table_new_full (g_str_hash, g_str_equal,
+                                                         NULL, (GDestroyNotify) g_object_unref);
 
-        gconf_client_add_dir (client, MOUSE_SETTINGS_DIR, GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
-        gconf_client_add_dir (client, GTK_SETTINGS_DIR, GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
-        gconf_client_add_dir (client, INTERFACE_SETTINGS_DIR, GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
-        gconf_client_add_dir (client, SOUND_SETTINGS_DIR, GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
-        gconf_client_add_dir (client, GTK_MODULES_DIR, GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
-        gconf_client_add_dir (client, FONT_RENDER_DIR, GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
+        g_hash_table_insert (manager->priv->settings,
+                             MOUSE_SETTINGS_SCHEMA, g_settings_new (MOUSE_SETTINGS_SCHEMA));
+        g_hash_table_insert (manager->priv->settings,
+                             INTERFACE_SETTINGS_SCHEMA, g_settings_new (INTERFACE_SETTINGS_SCHEMA));
+        g_hash_table_insert (manager->priv->settings,
+                             SOUND_SETTINGS_SCHEMA, g_settings_new (SOUND_SETTINGS_SCHEMA));
 
         for (i = 0; i < G_N_ELEMENTS (translations); i++) {
-                GConfValue *val;
-                GError     *err;
+                GVariant *val;
+                GSettings *settings;
 
-                err = NULL;
-                val = gconf_client_get (client,
-                                        translations[i].gconf_key,
-                                        &err);
-
-                if (err != NULL) {
-                        g_warning ("Error getting value for %s: %s",
-                                   translations[i].gconf_key,
-                                   err->message);
-                        g_error_free (err);
-                } else {
-                        process_value (manager, &translations[i], val);
-                        if (val != NULL) {
-                                gconf_value_free (val);
-                        }
+                settings = g_hash_table_lookup (manager->priv->settings,
+                                                translations[i].gsettings_schema);
+                if (settings == NULL) {
+                        g_warning ("Schemas '%s' has not been setup", translations[i].gsettings_schema);
+                        continue;
                 }
+
+                val = g_settings_get_value (settings, translations[i].gsettings_key);
+
+                process_value (manager, &translations[i], val);
         }
 
-        manager->priv->notify[0] =
-                register_config_callback (manager, client,
-                                          MOUSE_SETTINGS_DIR,
-                                          (GConfClientNotifyFunc) xsettings_callback);
-        manager->priv->notify[1] =
-                register_config_callback (manager, client,
-                                          GTK_SETTINGS_DIR,
-                                          (GConfClientNotifyFunc) xsettings_callback);
-        manager->priv->notify[2] =
-                register_config_callback (manager, client,
-                                          INTERFACE_SETTINGS_DIR,
-                                          (GConfClientNotifyFunc) xsettings_callback);
-        manager->priv->notify[3] =
-                register_config_callback (manager, client,
-                                          SOUND_SETTINGS_DIR,
-                                          (GConfClientNotifyFunc) xsettings_callback);
+        list = g_hash_table_get_values (manager->priv->settings);
+        for (l = list; l != NULL; l = l->next) {
+                g_signal_connect (G_OBJECT (l->data), "changed",
+                                  G_CALLBACK (xsettings_callback), manager);
+        }
+        g_list_free (list);
 
-        manager->priv->notify[4] =
-                register_config_callback (manager, client,
-                                          GTK_MODULES_DIR,
-                                          (GConfClientNotifyFunc) gtk_modules_callback);
-        gtk_modules_callback (client, 0, NULL, manager);
+        /* Plugin settings (GTK modules and Xft) */
+        manager->priv->plugin_settings = g_settings_new (XSETTINGS_PLUGIN_SCHEMA);
+        g_signal_connect (manager->priv->plugin_settings, "changed",
+                          G_CALLBACK (plugin_callback), manager);
 
-#ifdef HAVE_FONTCONFIG
-        manager->priv->notify[5] =
-                register_config_callback (manager, client,
-                                          FONT_RENDER_DIR,
-                                          (GConfClientNotifyFunc) xft_callback);
-        update_xft_settings (manager, client);
+        manager->priv->gtk = gsd_xsettings_gtk_new ();
+        g_signal_connect (G_OBJECT (manager->priv->gtk), "notify::gtk-modules",
+                          G_CALLBACK (gtk_modules_callback), manager);
+
+        /* Xft settings */
+        update_xft_settings (manager);
 
         start_fontconfig_monitor (manager);
-#endif /* HAVE_FONTCONFIG */
-
-        g_object_unref (client);
 
         for (i = 0; manager->priv->managers [i]; i++)
                 xsettings_manager_set_string (manager->priv->managers [i],
@@ -893,7 +739,6 @@ void
 gnome_xsettings_manager_stop (GnomeXSettingsManager *manager)
 {
         GnomeXSettingsManagerPrivate *p = manager->priv;
-        GConfClient *client;
         int i;
 
         g_debug ("Stopping xsettings manager");
@@ -906,27 +751,14 @@ gnome_xsettings_manager_stop (GnomeXSettingsManager *manager)
                 p->managers = NULL;
         }
 
-        client = gconf_client_get_default ();
-
-        gconf_client_remove_dir (client, MOUSE_SETTINGS_DIR, NULL);
-        gconf_client_remove_dir (client, GTK_SETTINGS_DIR, NULL);
-        gconf_client_remove_dir (client, INTERFACE_SETTINGS_DIR, NULL);
-        gconf_client_remove_dir (client, SOUND_SETTINGS_DIR, NULL);
-        gconf_client_remove_dir (client, GTK_MODULES_DIR, NULL);
-#ifdef HAVE_FONTCONFIG
-        gconf_client_remove_dir (client, FONT_RENDER_DIR, NULL);
-
+        g_object_unref (manager->priv->plugin_settings);
         stop_fontconfig_monitor (manager);
-#endif /* HAVE_FONTCONFIG */
 
-        for (i = 0; i < G_N_ELEMENTS (p->notify); ++i) {
-                if (p->notify[i] != 0) {
-                        gconf_client_notify_remove (client, p->notify[i]);
-                        p->notify[i] = 0;
-                }
-        }
+        g_hash_table_destroy (p->settings);
+        p->settings = NULL;
 
-        g_object_unref (client);
+        g_object_unref (p->gtk);
+        p->gtk = NULL;
 }
 
 static void
